"""
Network subsystem package
"""

# from connectionpool import BMConnectionPool
from .node import Peer
from .threads import StoppableThread


__all__ = [
    "adjust_announce", "Peer", "StoppableThread"
    # "AddrThread", "AnnounceThread", "BMNetworkThread", "Dandelion",
    # "DownloadThread", "InvThread", "ReceiveQueueThread", "UploadThread",
]


def adjust_announce(app, enable=True):
    """Start AnnounceThread or stop when state changes"""
    from .announcethread import AnnounceThread
    if enable:
        announceThread = AnnounceThread(app)
        announceThread.daemon = True
        announceThread.start()
    else:
        try:
            app.state.announceThread.stopThread()
        except AttributeError:
            pass


def start(app):
    """Start network threads"""
    from .addrthread import AddrThread
    from .connectionpool import BMConnectionPool
    from .dandelion import Dandelion
    from .downloadthread import DownloadThread
    from .invthread import InvThread
    from .networkthread import BMNetworkThread
    from .knownnodes import readKnownNodes
    from .receivequeuethread import ReceiveQueueThread
    from .uploadthread import UploadThread

    readKnownNodes()
    # init, needs to be early because other thread may access it early
    Dandelion()
    # init
    pool = BMConnectionPool(app)
    pool.connectToStream(1)
    asyncoreThread = BMNetworkThread(pool)
    asyncoreThread.daemon = True
    asyncoreThread.start()
    invThread = InvThread(app)
    invThread.daemon = True
    invThread.start()
    addrThread = AddrThread(app)
    addrThread.daemon = True
    addrThread.start()
    downloadThread = DownloadThread(app)
    downloadThread.daemon = True
    downloadThread.start()
    uploadThread = UploadThread(app)
    uploadThread.daemon = True
    uploadThread.start()

    # Optional components
    for i in range(app.config.getint('threads', 'receive')):
        receiveQueueThread = ReceiveQueueThread(app, i)
        receiveQueueThread.daemon = True
        receiveQueueThread.start()
    if app.config.safeGetBoolean('bitmessagesettings', 'udp'):
        adjust_announce(app, True)
