"""
Announce myself (node address)
"""
import time

from connectionpool import BMConnectionPool
from node import Peer
from threads import StatefulThread


class AnnounceThread(StatefulThread):
    """A thread to manage regular announcing of this node"""
    name = "Announcer"
    announceInterval = 60

    def run(self):
        lastSelfAnnounced = 0
        while not self._stopped and self.state.shutdown == 0:
            processed = 0
            if lastSelfAnnounced < time.time() - self.announceInterval:
                self.announceSelf()
                lastSelfAnnounced = time.time()
            if processed == 0:
                self.stop.wait(10)

    def announceSelf(self):
        """Announce our presence"""
        for connection in BMConnectionPool().udpSockets.values():
            if not connection.announcing:
                continue
            for stream in self.state.streamsInWhichIAmParticipating:
                addr = (stream, Peer(
                        '127.0.0.1',
                        self.config.safeGetInt('bitmessagesettings', 'port')),
                        time.time())
                connection.append_write_buf(
                    self.protocol.assembleAddrMessage([addr]))
